@@ -1,21 +1,33 @@
+from dexy.dexy_filter import DexyFilter
 from dexy.document import Document
-from dexy.handler import DexyHandler
 from dexy.reporter import Reporter
 from dexy.topological_sort import topological_sort
+from dexy.utils import AttrDict
 from inspect import isclass
 from ordereddict import OrderedDict
+import dexy.db
 import fnmatch
 import glob
 import json
 import logging
 import os
+import pprint
 import re
+import sqlite3
 import sre_constants
 import sys
 
 class Controller(object):
     def __init__(self):
         self.config = {}
+
+        # In actual usage, these initialized values are replaced by optparse
+        # Namespace or the argparse equivalent. These initialized values are
+        # used in testing, AttrDict is used so we can simulate the globals
+        # attr.
+        self.args = AttrDict({})
+        self.args.globals = {}
+
         self.install_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))
         if not hasattr(self, 'log'):
             self.log = logging.getLogger('dexy')
@@ -56,7 +68,7 @@ class Controller(object):
         self.reports_dirs = [r.REPORTS_DIR for r in reporters]
         return reporters
 
-    def find_handlers(self):
+    def find_filters(self):
         sys.path.append(os.curdir)
 
         handler_dirs = []
@@ -72,8 +84,8 @@ class Controller(object):
 
         handlers = {}
 
-        for a in DexyHandler.ALIASES:
-            handlers[a] = DexyHandler
+        for a in DexyFilter.ALIASES:
+            handlers[a] = DexyFilter
 
         for d in handler_dirs:
             self.log.info("Automatically loading all filters found in %s" % d)
@@ -100,12 +112,15 @@ class Controller(object):
 
                     for k in dir(mod):
                         klass = mod.__dict__[k]
-                        if isclass(klass) and not (klass == DexyHandler) and issubclass(klass, DexyHandler):
+                        if isclass(klass) and not (klass == DexyFilter) and issubclass(klass, DexyFilter):
                             if not klass.ALIASES:
                                 self.log.info("class %s is not available because it has no aliases" % klass.__name__)
                             elif not klass.executable_present():
                                 self.log.info("class %s is not available because %s not found" %
                                               (klass.__name__, klass.executable()))
+                            elif not klass.enabled():
+                                self.log.info("class %s is not available because it is not enabled" %
+                                              (klass.__name__))
                             else:
                                 for a in klass.ALIASES:
                                     if handlers.has_key(a):
@@ -115,8 +130,17 @@ class Controller(object):
             self.log.info("...finished loading filters from %s" % d)
         return handlers
 
-    def register_handlers(self):
-        self.handlers = self.find_handlers()
+    def register_filters(self):
+        self._filters = self.find_filters()
+
+    def get_handler_for_alias(self, alias):
+        """Given an alias, return the corresponding handler."""
+        if self._filters.has_key(alias):
+            return self._filters[alias]
+        elif alias.startswith("-") or alias.startswith("alias-") or alias.startswith("al-") or alias in ['al', 'alias']:
+            self._filters[alias] = DexyFilter
+        else:
+            raise Exception("You requested filter alias '%s' but this is not available." % alias)
 
     def register_reporters(self):
         self.reporters = self.find_reporters()
@@ -126,18 +150,27 @@ class Controller(object):
         relative_path = os.path.relpath(path_to_dir, rel_to)
         path_elements = relative_path.split(os.sep)
         config_dict = {}
+        global_args = {}
         for i in range(0,len(path_elements)+1):
             config_file_path_elements = [rel_to] + path_elements[0:i] + [self.config_file]
             config_filename = os.path.join(*config_file_path_elements)
-            if os.path.exists(config_filename):
-                self.log.info("loading config %s" % config_filename)
-                config_file = open(config_filename, "r")
+            config_files = glob.glob(config_filename)
+            for cf in config_files:
+                self.log.info("loading config %s" % cf)
+                config_file = open(cf, "r")
                 try:
                     json_dict = json.load(config_file)
                 except ValueError as e:
                     raise Exception("""Your config file %s has invalid JSON. Details: %s""" %
-                                    (config_filename, e.message))
-                config_dict.update(json_dict)
+                                    (cf, e.message))
+                if json_dict.has_key("$reset"):
+                    config_dict = json_dict
+                else:
+                    config_dict.update(json_dict)
+
+                if json_dict.has_key("$globals"):
+                    global_args.update(json_dict["$globals"])
+                config_dict['$globals'] = global_args
         self.config[path_to_dir] = config_dict
 
     def process_config(self):
@@ -249,7 +282,11 @@ Please pass a valid Python-style regular expression for
 'except', NOT a glob-style matcher. Error message from
 re.compile: %s""" % (args['except'], e))
                     if re.match(except_re, f):
-                        print "skipping %s as it matches except pattern %s" % (f, args['except'])
+                        print "skipping %s for %s as it matches except pattern %s" % (
+                                f,
+                                input_directive,
+                                args['except']
+                                )
                         create = False
 
                 if create:
@@ -282,6 +319,7 @@ re.compile: %s""" % (args['except'], e))
                     if not hasattr(doc, 'args'):
                         doc.args = args
 
+                    doc.controller_args = self.args
                     self.members[key] = doc
                     docs.append(doc) # just a local list
             return docs
@@ -297,11 +335,26 @@ re.compile: %s""" % (args['except'], e))
         self.depends = []
 
         # Create Document objects for all docs.
-        self.log.debug("About to process config\n")
-        self.log.debug(self.config)
-        for path, config in self.config.items():
-            for k, v in config.items():
-                parse_doc(path, k, v)
+        self.log.debug("About to process config:\n" + pprint.pformat(self.config))
+        for path, config in self.config.iteritems():
+            ### @export "features-global-args-1"
+            if config.has_key("$globals"):
+                global_args = config["$globals"]
+            else:
+                global_args = {}
+
+            if self.args.globals:
+                global_args.update(self.args.globals)
+
+            for k, v in config.iteritems():
+                local_args = global_args.copy()
+                local_args.update(v)
+                for kg in global_args.keys():
+                    if local_args.has_key(kg):
+                        if isinstance(local_args[kg], dict):
+                            local_args[kg].update(global_args[kg])
+                parse_doc(path, k, local_args)
+            ### @end
 
         # Determine dependencies
         for doc in self.members.values():
@@ -310,6 +363,14 @@ re.compile: %s""" % (args['except'], e))
                 depend(doc, input_doc)
 
         ordering = topological_sort(range(len(self.members)), self.depends)
+
+        self.db = None
+        self.batch_id = None
+        if dexy.db.DB_AVAILABLE:
+            print "initializing database"
+            self.db = dexy.db.Db()
+            self.batch_id = self.db.next_batch_id()
+
         ordered_members = OrderedDict()
         for i in ordering:
             key = self.members.keys()[i]
@@ -322,8 +383,7 @@ re.compile: %s""" % (args['except'], e))
     def setup_and_run(self):
         if not hasattr(self, 'config_file'):
             self.config_file = '.dexy'
-        self.register_handlers()
-        self.register_reporters()
+        self.register_filters()
         self.process_config()
         self.docs = self.run()
         return self.docs
