@@ -1,16 +1,15 @@
+from dexy.utils import profile_memory
 import StringIO
-import glob
 import json
 import logging
 import os
-import sys
 import urllib2
-import traceback
 
 class Document(object):
     def __init__(self, artifact_class, name_or_key, filters = []):
         self.artifact_class = artifact_class
         self.name = name_or_key.split("|")[0]
+        self.ext = os.path.splitext(self.name)[1]
         self.filters = name_or_key.split("|")[1:]
         self.filters += filters
         self.inputs = []
@@ -35,10 +34,7 @@ class Document(object):
         self.use_all_inputs = False
 
     def final_artifact(self):
-        if len(self.artifacts) == 0:
-            raise Exception("Trying to retrieve final artifact when artifacts are empty!")
-
-        return self.artifacts[-1]
+        return self.last_artifact
 
     def key(self):
         return "%s|%s" % (self.name, "|".join(self.filters))
@@ -57,15 +53,16 @@ class Document(object):
         else:
             self.inputs = [members_dict[k] for k in self.input_keys]
 
-    def next_handler_name(self):
+    def next_filter_alias(self):
         if self.at_last_step():
-            return 'None'
+            return None
         else:
             return self.filters[self.step]
 
-    def next_handler_class(self):
-        if not self.at_last_step():
-            return self.controller.handlers[self.next_handler_name()]
+    def next_filter_class(self):
+        alias = self.next_filter_alias()
+        if alias:
+            return self.controller.get_handler_for_alias(alias)
 
     def at_last_step(self):
         return (len(self.filters) == self.step)
@@ -74,7 +71,10 @@ class Document(object):
         input_artifacts = {}
         for i in self.inputs:
             k = i.key()
-            input_artifacts[k] = i.final_artifact()
+            a = i.final_artifact()
+            if not a:
+                raise Exception("No final artifact for document %s" % i.key())
+            input_artifacts[k] = a
         return input_artifacts
 
     def initial_artifact_data(self):
@@ -169,69 +169,49 @@ class Document(object):
         return data
 
     def create_initial_artifact(self):
+        # Create and set up the new artifact.
         artifact = self.artifact_class.setup(self, self.name)
-        if os.path.basename(self.name).startswith("_"):
-            artifact.final = False
-        artifact.set_data(self.initial_artifact_data())
-        artifact.set_hashstring()
-        artifact.save()
+        artifact.elapsed = 0
+        artifact.document_key = self.key()
+        artifact.controller_args = self.controller_args
+
+        # Add the new artifact to the document's list of artifacts.
         self.artifacts.append(artifact)
+
         return artifact
 
     def run(self, controller):
         self.controller = controller
         self.step = 0
 
-        self.log.info("starting to process document")
-        self.log.info("args:")
-        for k, v in self.args.items():
-            self.log.info("%s: %s" % (k, v))
-        self.log.info("list of inputs:")
-        for i in self.inputs:
-            self.log.info(i.key())
-
         artifact = self.create_initial_artifact()
         artifact_key = artifact.key
-        self.log.info("(step %s) %s -> %s" % \
+        self.log.info("(step %s) [run] %s -> %s" % \
                  (self.step, artifact_key, artifact.filename()))
-
+        profile_memory("document-%s-step-%s" % (self.key(), self.step))
+        self.last_artifact = artifact
         for f in self.filters:
             previous_artifact = artifact
             artifact_key += "|%s" % f
             self.step += 1
 
-            if not self.controller.handlers.has_key(f):
-                raise Exception("""You requested filter alias '%s'
-                                but this is not available.""" % f)
-            HandlerClass = self.controller.handlers[f]
-            h = HandlerClass.setup(
-                self,
-                artifact_key,
-                previous_artifact,
-                self.next_handler_class()
-            )
+            FilterClass = self.controller.get_handler_for_alias(f)
+            artifact = self.artifact_class.setup(self, artifact_key, FilterClass, previous_artifact)
 
-            try:
-                artifact = h.generate_artifact()
-            except Exception as e:
-                print "Error occurred while applying", f, "for", artifact_key
-                x, y, tb = sys.exc_info()
-                print "Original traceback:"
-                traceback.print_tb(tb)
-                if hasattr(h.artifact, 'hashstring'):
-                    pattern = os.path.join(self.controller.artifacts_dir, h.artifact.hashstring)
-                    files_matching = glob.glob(pattern)
-                    if len(files_matching) > 0:
-                        print "Here are working files which might have clues about this error:"
-                        for f in files_matching:
-                            print f
-                raise e
-
-            if not artifact:
-                raise Exception("no artifact created!")
-            self.artifacts.append(artifact)
-
-            self.log.info("(step %s) %s -> %s" % \
+            self.log.info("(step %s) [run] %s -> %s" % \
                      (self.step, artifact_key, artifact.filename()))
 
+            artifact.run()
+
+            self.last_artifact = artifact
+      	    profile_memory("document-%s-step-%s" % (self.key(), self.step))
+
+
+        # Make sure all additional inputs are saved.
+        for k, a in artifact._inputs.iteritems():
+            if not a.is_complete():
+                a.state = 'complete'
+                a.save()
+
+        profile_memory("document-%s-complete" % (self.key()))
         return self
