@@ -1,8 +1,10 @@
 from dexy.artifact import Artifact
 from dexy.controller import Controller
+from dexy.utils import profile_memory
 from dexy.version import VERSION
 from inspect import isclass
 from logging.handlers import RotatingFileHandler
+import cProfile
 import logging
 import os
 import re
@@ -25,6 +27,7 @@ level of your project: %s.
 Any directory with a file named .nodexy will be skipped, and subdirectories of this will also be skipped.
 """ % (", ".join(EXCLUDE_DIRS_ALL_LEVELS), ", ".join(EXCLUDE_DIRS_ROOT))
 
+
 def artifact_class(artifact_class_name, log):
     artifact_classes = {}
 
@@ -72,6 +75,27 @@ def setup_option_parser():
         option_parser = 'argparse'
         parser = argparse.ArgumentParser()
 
+
+        class GlobalsArgparseAction(argparse.Action):
+            def __call__(self, parser, namespace, values, option_string=None):
+                value_dict = getattr(namespace, self.dest)
+                if not value_dict:
+                    value_dict = {}
+                if not "=" in values:
+                    raise Exception("args passed to %s should be in KEY=value format, no '=' found in %s" % (option_string, values))
+                k, v = values.split("=")
+                value_dict[k] = v
+                setattr(namespace, self.dest, value_dict)
+
+        parser.add_argument(
+            '--global',
+            help="""Use to set a global variable which will be available within dexy tasks,
+            use KEY=value syntax and specify option multiple times if you need to specify more than 1 key value pair
+            e.g. VERSION=0.0.1""",
+            action=GlobalsArgparseAction,
+            dest='globals'
+        )
+
         parser.add_argument(
             'dir',
             help='directory of files to process with dexy',
@@ -84,6 +108,12 @@ def setup_option_parser():
             nargs='+'
         )
 
+        parser.add_argument(
+            '--use-reporters', '--use-reporter',
+            help="list reporters to be run",
+            nargs='+'
+        )
+
         parser.add_argument(
             '-v', '--version',
             action='version',
@@ -100,6 +130,11 @@ def setup_option_parser():
             help=EXCLUDE_HELP + ' Separate multiple directories with commas.'
         )
 
+        parser.add_option(
+            "--use-reporters",
+            help="list reporters to be used. separate multiple reproters with commas"
+        )
+
     # Remaining options are the same for argparse and optparse
     add_option(parser,
         '-n', '--no-recurse',
@@ -109,6 +144,14 @@ def setup_option_parser():
         help='do not recurse into subdirectories (default: recurse)'
     )
 
+    add_option(parser,
+        '--no-tests',
+        dest='run_tests',
+        default=True,
+        action='store_false',
+        help='do not run any dexy tests (using the test filter) - helpful for populating test content'
+    )
+
     add_option(parser,
         '-u', '--utf8',
         default=False,
@@ -137,6 +180,13 @@ def setup_option_parser():
         help='list all available filters (does not run dexy)'
     )
 
+    add_option(parser,
+        '--profile',
+        default=False,
+        action='store_true',
+        help='run dexy with cProfile'
+    )
+
     add_option(parser,
         '--reporters',
         default=False,
@@ -227,6 +277,11 @@ def setup_option_parser():
         else:
             additional_excludes = []
 
+        if args.use_reporters:
+            use_reporters = args.use_reporters
+        else:
+            use_reporters = None
+
     elif (option_parser == 'optparse'):
         (args, argv) = parser.parse_args()
         if len(argv) == 0:
@@ -237,6 +292,11 @@ def setup_option_parser():
             additional_excludes = args.exclude.split(',')
         else:
             additional_excludes = []
+
+        if args.use_reporters:
+            use_reporters = args.use_reporters.split(',')
+        else:
+            use_reporters = None
     else:
         raise Exception("unexpected option_parser %s" % option_parser)
 
@@ -278,17 +338,17 @@ def setup_option_parser():
     if not os.path.exists(args.artifacts_dir):
         path_to_artifacts_dir = os.path.join(project_base, args.artifacts_dir)
         raise Exception(
-            """artifacts directory not found at %s,
-            please create a directory called %s if you want to use
-            this location as a dexy project root""" % (path_to_artifacts_dir, args.artifacts_dir)
+            """artifacts directory not found at '%s',
+            please call dexy with --setup if you want to use
+            this location as a dexy project root""" % (args.artifacts_dir)
         )
 
     if not os.path.exists(args.logs_dir):
         path_to_logs_dir = os.path.join(project_base, args.logs_dir)
         raise Exception(
             """logs directory not found at %s,
-            please create a directory called %s if you want to use
-            this location as a dexy project root""" % (path_to_logs_dir, args.logs_dir)
+            please call dexy with --setup if you want to use
+            this location as a dexy project root""" % (path_to_logs_dir)
         )
 
     # Set up main dexy log
@@ -330,6 +390,12 @@ def setup_option_parser():
             print "purging contents of %s" % args.logs_dir
             shutil.rmtree(args.logs_dir)
 
+        if os.path.exists('testdb') and False:
+            os.remove('testdb')
+
+        if os.path.exists('output-latest.tgz'):
+            os.remove('output-latest.tgz')
+
         controller = Controller()
         controller.find_reporters()
         for d in controller.reports_dirs:
@@ -367,9 +433,9 @@ def setup_option_parser():
     if args.filters:
         args.run_dexy = False
         controller = Controller()
-        handlers = controller.find_handlers()
-        for k in sorted(handlers.keys()):
-            klass = handlers[k]
+        filters = controller.find_filters()
+        for k in sorted(filters.keys()):
+            klass = filters[k]
             print
             print k, ":", klass.__name__
             if klass.executable():
@@ -407,7 +473,12 @@ def setup_controller():
     controller.log = log
     controller.config_file = args.config
     controller.use_local_files = args.local
-    controller.find_reporters()
+    reporters = controller.find_reporters()
+    if args.use_reporters:
+        controller.reporters = [r for r in reporters if r.__name__ in args.use_reporters]
+    else:
+        controller.reporters = [r for r in reporters if r.DEFAULT]
+
     controller.additional_excludes = additional_excludes
 
     return controller, args, log
@@ -485,10 +556,15 @@ def dexy_command():
         log.info("processing dir %s" % controller.dir_name)
         controller.load_config(controller.dir_name)
 
-    controller.setup_and_run()
+    if args.profile:
+        cProfile.runctx("controller.setup_and_run()", globals(), locals(), "dexy.prof")
+    else:
+        controller.setup_and_run()
+
     if not args.no_reports:
         for reporter_klass in controller.reporters:
             reporter_klass(controller).run()
+            profile_memory("report-%s-complete" % reporter_klass.__name__)
     else:
         print 'reports not run'
 
