@@ -1,111 +1,142 @@
-from dexy.handler import DexyHandler
+from dexy.dexy_filter import DexyFilter
 from ordereddict import OrderedDict
+import simplejson as json
 import os
 import pexpect
+import re
+import sys
 import time
 
-class ProcessLinewiseInteractiveHandler(DexyHandler):
+class ProcessLinewiseInteractiveHandler(DexyFilter):
     """
     Intended for use with interactive processes, such as python interpreter,
     where your goal is to have a session transcript divided into same sections
     as input. Sends input line-by-line.
     """
-    EXECUTABLE = 'python'
-    PROMPT = '>>>|\.\.\.' # Python uses >>> prompt normally and ... when in multi-line structures like loops
-    INPUT_EXTENSIONS = [".txt", ".py"]
-    OUTPUT_EXTENSIONS = [".pycon"]
-    ALIASES = ['pycon']
+    PROMPT = ['>>>', '...'] # Python uses >>> prompt normally and ... when in multi-line structures like loops
+    TRIM_PROMPT = '>>>'
+    LINE_ENDING = "\r\n"
     IGNORE_ERRORS = False # Allow overriding default per-handler.
+    SAVE_VARS_TO_JSON_CMD = None
+    ALIASES = None
 
     def process_dict(self, input_dict):
         output_dict = OrderedDict()
-        if self.doc.args.has_key('timeout'):
-            timeout = self.doc.args['timeout']
+
+        if self.artifact.args.has_key('timeout'):
+            timeout = self.artifact.args['timeout']
             self.log.info("using custom timeout %s for %s" % (timeout, self.artifact.key))
         else:
             timeout = None
-        if self.doc.args.has_key('env'):
+        if self.artifact.args.has_key('env'):
             env = os.environ
-            env.update(self.doc.args['env'])
-            self.log.info("adding to env: %s" % self.artifact.doc.args['env'])
+            env.update(self.artifact.args['env'])
+            self.log.info("adding to env: %s" % self.artifact.args['env'])
         else:
             env = None
 
         proc = pexpect.spawn(self.EXECUTABLE, cwd=self.artifact.artifacts_dir, env=env)
-        proc.expect(self.PROMPT)
+        proc.expect_exact(self.PROMPT, timeout=timeout)
         start = (proc.before + proc.after)
 
+        search_terms = ["%s%s" % (self.LINE_ENDING, p) for p in self.PROMPT]
+
         for k, s in input_dict.items():
             # TODO Should stop processing if an error is raised.
             section_transcript = start
             start = ""
-            for l in s.split("\n"):
+            for l in (s+"\r\n").splitlines():
                 section_transcript += start
                 start = ""
                 proc.sendline(l)
-                proc.expect(self.PROMPT, timeout=timeout)
+                proc.expect_exact(search_terms, timeout=timeout)
                 section_transcript += proc.before
                 start = proc.after
-            output_dict[k] = section_transcript
+
+            # Strip blank lines/trailing prompts at end of section
+            lines = section_transcript.split(self.LINE_ENDING)
+            while len(lines) > 0 and re.match("^\s*(%s)\s*$|^\s*$" % self.TRIM_PROMPT, lines[-1]):
+                lines = lines[0:-1]
+
+#            output_dict[k] = ''.join([c for c in self.LINE_ENDING.join(lines) if ord(c) > 31 or ord(c) in [9, 10]])
+            output_dict[k] = self.LINE_ENDING.join(lines)
+
+        record_vars = self.artifact.args.has_key('record_vars') and self.artifact.args['record_vars']
+        if record_vars:
+            if not self.SAVE_VARS_TO_JSON_CMD:
+                raise Exception("Can't record vars since SAVE_VARS_TO_JSON_CMD not set.")
+            artifact = self.artifact.add_additional_artifact(self.artifact.key + "-vars", 'json')
+            cmd = self.SAVE_VARS_TO_JSON_CMD % artifact.filename()
+
+            section_transcript = start
+            start = ""
+            for l in cmd.splitlines():
+                section_transcript += start
+                start = ""
+                proc.sendline(l)
+                proc.expect_exact(search_terms, timeout=timeout)
+                section_transcript += proc.before
+                start = proc.after
+
+            output_dict['dexy--save-vars'] = section_transcript
+
         try:
             proc.close()
         except pexpect.ExceptionPexpect:
             print "process %s may not have closed" % proc.pid
 
         if proc.exitstatus is not None and proc.exitstatus != 0:
-            if not (self.IGNORE_ERRORS or self.doc.controller.args.ignore_errors):
+            if not (self.IGNORE_ERRORS or self.artifact.args.ignore_errors):
                 raise Exception("""proc returned nonzero status code! if you don't
 want dexy to raise errors on failed scripts then pass the --ignore-errors option""")
         return output_dict
 
-class ProcessSectionwiseInteractiveHandler(DexyHandler):
+class PythonLinewiseInteractiveHandler(ProcessLinewiseInteractiveHandler):
+    EXECUTABLE = 'python'
+    VERSION = 'python --version'
+    INPUT_EXTENSIONS = [".txt", ".py"]
+    OUTPUT_EXTENSIONS = [".pycon"]
+    ALIASES = ['pycon']
+    SAVE_VARS_TO_JSON_CMD = """dexy__vars_file = open("%s", "w")
+dexy__x = {}
+for dexy__k, dexy__v in locals().items():
+    dexy__x[dexy__k] = str(dexy__v)
+
+json.dump(dexy__x, dexy__vars_file)
+dexy__vars_file.close()
+"""
+
+class RLinewiseInteractiveHandler(ProcessLinewiseInteractiveHandler):
     """
-    Intended for use with interactive processes, such as R interpreter,
-    where your goal is to have a session transcript divided into same sections
-    as input. Sends input section-by-section.
+    Runs R
     """
-    EXECUTABLE = 'R --quiet --vanilla'
+    EXECUTABLE = "R --quiet --vanilla"
     VERSION = "R --version"
-    PROMPT = '>'
-    COMMENT = '#'
-    TRAILING_PROMPT = "\r\n> "
     INPUT_EXTENSIONS = ['.txt', '.r', '.R']
     OUTPUT_EXTENSIONS = ['.Rout']
+    PROMPT = [">", "+"]
+    TRIM_PROMPT = ">"
     ALIASES = ['r', 'rint']
+    SAVE_VARS_TO_JSON_CMD = """
+if ("rjson" %%in%% installed.packages()) {
+    library(rjson)
+    dexy__json_file <- file("%s", "w")
+    writeLines(toJSON(as.list(environment())), dexy__json_file)
+    close(dexy__json_file)
+} else {
+   cat("Can't automatically save environment to JSON since rjson package not installed.")
+}
+"""
 
-    def process_dict(self, input_dict):
-        output_dict = OrderedDict()
-
-        if self.artifact.doc.args.has_key('timeout'):
-            timeout = self.artifact.doc.args['timeout']
-            self.log.info("using custom timeout %s for %s" % (timeout, self.artifact.key))
-        else:
-            timeout = None
-        if self.artifact.doc.args.has_key('env'):
-            env = os.environ
-            env.update(self.artifact.doc.args['env'])
-            self.log.info("adding to env: %s" % self.artifact.doc.args['env'])
-        else:
-            env = None
-
-        proc = pexpect.spawn(self.EXECUTABLE,
-                             cwd=self.artifact.artifacts_dir,
-                             env=env)
-        proc.expect(self.PROMPT)
-        start = (proc.before + proc.after)
-
-        for k, s in input_dict.items():
-            section_transcript = start
-            start = ""
-            proc.send(s)
-            proc.sendline(self.COMMENT * 5)
-
-            proc.expect(self.COMMENT * 5, timeout = timeout)
-
-            section_transcript += proc.before.rstrip(self.TRAILING_PROMPT)
-            output_dict[k] = ''.join([c for c in section_transcript if ord(c) > 31 or ord(c) in [9, 10]])
-
-        return output_dict
+class RhinoInteractiveHandler(ProcessLinewiseInteractiveHandler):
+    """
+    Runs rhino JavaScript interpeter.
+    """
+    EXECUTABLE = "rhino"
+    INPUT_EXTENSIONS = [".js"]
+    OUTPUT_EXTENSIONS = [".txt"]
+    ALIASES = ['jsint', 'rhino']
+    PROMPT = "js> "
 
 class ClojureInteractiveHandler(ProcessLinewiseInteractiveHandler):
     """
@@ -118,7 +149,7 @@ class ClojureInteractiveHandler(ProcessLinewiseInteractiveHandler):
     ALIASES = ['clj', 'cljint']
     PROMPT = "user=> "
 
-class ProcessTimingHandler(DexyHandler):
+class ProcessTimingHandler(DexyFilter):
     """
     Runs python code N times and reports timings.
     """
@@ -138,7 +169,7 @@ class ProcessTimingHandler(DexyHandler):
             times.append("%s" % (time.time() - start))
         self.artifact.data_dict['1'] = "\n".join(times)
 
-class Pdf2Jpg(DexyHandler):
+class Pdf2Jpg(DexyFilter):
     """
     Converts a PDF file to JPEG format.
     """
@@ -158,7 +189,7 @@ class Pdf2Jpg(DexyHandler):
         self.log.debug(command)
         self.artifact.stdout = pexpect.run(command, cwd=self.artifact.artifacts_dir)
 
-class Pdf2Png(DexyHandler):
+class Pdf2Png(DexyFilter):
     """
     Converts a PDF file to PNG format.
     """
@@ -178,7 +209,7 @@ class Pdf2Png(DexyHandler):
         self.log.debug(command)
         self.artifact.stdout = pexpect.run(command, cwd=self.artifact.artifacts_dir)
 
-class Ps2Pdf(DexyHandler):
+class Ps2Pdf(DexyFilter):
     """
     Converts a PS file to PDF format.
     """
@@ -196,80 +227,3 @@ class Ps2Pdf(DexyHandler):
         self.log.debug(command)
         self.artifact.stdout = pexpect.run(command, cwd=self.artifact.artifacts_dir)
 
-class VoiceHandler(DexyHandler):
-    """
-    Use text-to-speech to generate mp3 file from a text file. Uses whichever of
-    say, aiff, espeak, wav tools is found, then lame to convert to mp3.
-    """
-    INPUT_EXTENSIONS = [".*"]
-    OUTPUT_EXTENSIONS = [".mp3"]
-    ALIASES = ['voice', 'say']
-
-    def process(self):
-        self.artifact.generate_workfile()
-        work_file = os.path.basename(self.artifact.work_filename())
-        artifact_file = os.path.basename(self.artifact.filename())
-
-        for e, ext in {"say" : "aiff", "espeak" : "wav"}.items():
-            sound_file = artifact_file.replace('mp3', ext)
-            # TODO replace pexpect with subprocess
-            tts_bin, s = pexpect.run("which %s" % e, withexitstatus = True)
-            if s == 0:
-                self.log.info("%s text-to-speech found at %s" % (e, tts_bin))
-                break
-            else:
-                self.log.info("%s text-to-speech not found" % e)
-                e = None
-
-        if e == "say":
-            command = "say -f %s -o %s" % (work_file, sound_file)
-        elif e == "espeak":
-            command = "espeak -f %s -w %s" % (work_file, sound_file)
-        else:
-            raise Exception("unknown text-to-speech command %s" % e)
-
-        self.log.info(command)
-        self.artifact.stdout = pexpect.run(command, cwd=self.artifact.artifacts_dir)
-
-        # Converting to mp3
-        command = "lame %s %s" % (sound_file, artifact_file)
-        self.log.info(command)
-        self.artifact.stdout = pexpect.run(command, cwd=self.artifact.artifacts_dir)
-
-class RagelRubyHandler(DexyHandler):
-    """
-    Runs ragel for ruby.
-    """
-    INPUT_EXTENSIONS = [".rl"]
-    OUTPUT_EXTENSIONS = [".rb"]
-    ALIASES = ['rlrb', 'ragelruby']
-    VERSION = 'ragel --version'
-    EXECUTABLE = 'ragel -R'
-
-    def process(self):
-        self.artifact.generate_workfile()
-        artifact_file = self.artifact.filename(False)
-        work_file = self.artifact.work_filename(False)
-        command = "%s -o %s %s" % (self.executable(), artifact_file, work_file)
-        self.log.info(command)
-        self.artifact.stdout = pexpect.run(command, cwd=self.artifact.artifacts_dir)
-        self.artifact.data_dict['1'] = open(self.artifact.filename(), "r").read()
-
-class RagelRubyDotHandler(DexyHandler):
-    """
-    Generates state chart in .dot format of ragel state machine for ruby.
-    """
-    INPUT_EXTENSIONS = [".rl"]
-    OUTPUT_EXTENSIONS = [".dot"]
-    ALIASES = ['rlrbd', 'ragelrubydot']
-    VERSION = 'ragel --version'
-    EXECUTABLE = 'ragel -R'
-
-    def process(self):
-        self.artifact.generate_workfile()
-        wf = self.artifact.work_filename(False)
-        command = "%s -V %s" % (self.executable(), wf)
-        self.log.info(command)
-        ad = self.artifact.artifacts_dir
-        self.artifact.data_dict['1'] = pexpect.run(command, cwd=ad)
-
